from datetime import datetime
import time
from sqlalchemy import Column, Integer, String, DateTime
from sqlalchemy.exc import OperationalError
from models.base import Base, Session, logger

class MediaCounter(Base):
    __tablename__ = 'media_counters'
    
    id = Column(Integer, primary_key=True)
    chat_id = Column(String(50))
    image_count = Column(Integer, default=0)
    video_count = Column(Integer, default=0)
    last_updated = Column(DateTime, default=datetime.utcnow)

    @classmethod
    def increment_counter(cls, media_type):
        """Increment media counter with proper locking and error handling"""
        session = Session()
        max_retries = 3
        base_delay = 0.1
        
        for attempt in range(max_retries):
            try:
                # Get counter with lock
                counter = session.query(cls).with_for_update().first()
                
                if not counter:
                    counter = cls(image_count=0, video_count=0)
                    session.add(counter)
                    session.flush()
                
                # Normalize media type
                media_type = media_type.lower()
                
                # Increment appropriate counter
                if media_type == "image":
                    counter.image_count = (counter.image_count or 0) + 1
                elif media_type == "video":
                    counter.video_count = (counter.video_count or 0) + 1
                else:
                    raise ValueError(f"Invalid media type: {media_type}")
                
                counter.last_updated = datetime.utcnow()
                session.commit()
                return True
                
            except OperationalError as e:
                if "database is locked" in str(e) and attempt < max_retries - 1:
                    delay = base_delay * (2 ** attempt)
                    time.sleep(delay)
                    session.rollback()
                    continue
                session.rollback()
                raise e
            except Exception as e:
                session.rollback()
                logger.error(f"Error incrementing media counter: {str(e)}")
                raise e
            finally:
                if attempt == max_retries - 1:
                    session.close()
        
        return False

    @classmethod
    def get_counts(cls):
        session = Session()
        try:
            counter = cls.query.first()
            if not counter:
                return {"image": 0, "video": 0}
            return {
                "image": counter.image_count or 0,
                "video": counter.video_count or 0
            }
        finally:
            session.close()
